@@ -1,7 +1,7 @@
 /* infokey.c -- compile ~/.infokey to ~/.info.
-   $Id: infokey.c,v 1.1.1.1 2003/12/06 01:32:48 karl Exp $
+   $Id: infokey.c,v 1.7 2004/07/24 18:12:38 karl Exp $
 
-   Copyright (C) 1999, 2001, 2002, 2003 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -69,9 +69,16 @@ struct sect
   };
 
 /* Some "forward" declarations. */
-static char *mkpath ();
-static int compile (), write_infokey_file ();
-static void syntax_error (), error_message (), suggest_help (), short_help ();
+static char *mkpath (const char *dir, const char *file);
+static int compile (FILE *fp, const char *filename, struct sect *sections);
+static int write_infokey_file (FILE *fp, struct sect *sections);
+static void syntax_error (const char *filename,
+    unsigned int linenum, const char *fmt,
+    const void *a1, const void *a2, const void *a3, const void *a4);
+static void error_message (int error_code, const char *fmt,
+    const void *a1, const void *a2, const void *a3, const void *a4);
+static void suggest_help (void);
+static void short_help (void);
 
 
 /* **************************************************************** */
@@ -81,9 +88,7 @@ static void syntax_error (), error_message (), suggest_help (), short_help ();
 /* **************************************************************** */
 
 int
-main (argc, argv)
-     int argc;
-     char **argv;
+main (int argc, char **argv)
 {
   int getopt_long_index;	/* Index returned by getopt_long (). */
 
@@ -92,9 +97,11 @@ main (argc, argv)
   setlocale (LC_ALL, "");
 #endif
 
+#ifdef ENABLE_NLS
   /* Set the text message domain.  */
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
+#endif
 
   while (1)
     {
@@ -160,7 +167,8 @@ For more information about these matters, see the files named COPYING.\n"),
     }
   else if (optind != argc)
     {
-      error_message (0, _("incorrect number of arguments"));
+      error_message (0, _("incorrect number of arguments"),
+          NULL, NULL, NULL, NULL);
       suggest_help ();
       xexit (1);
     }
@@ -190,7 +198,8 @@ For more information about these matters, see the files named COPYING.\n"),
     inf = fopen (input_filename, "r");
     if (!inf)
       {
-	error_message (errno, _("cannot open input file `%s'"), input_filename);
+	error_message (errno, _("cannot open input file `%s'"),
+            input_filename, NULL, NULL, NULL);
 	xexit (1);
       }
 
@@ -203,7 +212,8 @@ For more information about these matters, see the files named COPYING.\n"),
 	outf = fopen (output_filename, FOPEN_WBIN);
 	if (!outf)
 	  {
-	    error_message (errno, _("cannot create output file `%s'"), output_filename);
+	    error_message (errno, _("cannot create output file `%s'"),
+                output_filename, NULL, NULL, NULL);
 	    xexit (1);
 	  }
 
@@ -213,12 +223,14 @@ For more information about these matters, see the files named COPYING.\n"),
 	write_error = 0;
 	if (!write_infokey_file (outf, sections))
 	  {
-	    error_message (errno, _("error writing to `%s'"), output_filename);
+	    error_message (errno, _("error writing to `%s'"),
+                output_filename, NULL, NULL, NULL);
 	    write_error = 1;
 	  }
 	if (fclose (outf) == EOF)
 	  {
-	    error_message (errno, _("error closing output file `%s'"), output_filename);
+	    error_message (errno, _("error closing output file `%s'"),
+                output_filename, NULL, NULL, NULL);
 	    write_error = 1;
 	  }
 	if (write_error)
@@ -236,9 +248,7 @@ For more information about these matters, see the files named COPYING.\n"),
 }
 
 static char *
-mkpath (dir, file)
-     const char *dir;
-     const char *file;
+mkpath (const char *dir, const char *file)
 {
   char *p;
 
@@ -365,21 +375,19 @@ mkpath (dir, file)
 	following the '=' is not ignored.
  */
 
-static int add_to_section (), lookup_action ();
+static int add_to_section (struct sect *s, const char *str, unsigned int len);
+static int lookup_action (const char *actname);
 
 /* Compile the input file into its various sections.  Return true if no
    error was encountered.
  */
 static int
-compile (fp, filename, sections)
-     FILE *fp;
-     const char *filename;
-     struct sect sections[];
+compile (FILE *fp, const char *filename, struct sect *sections)
 {
   int error = 0;
   char rescan = 0;
   unsigned int lnum = 0;
-  int c;
+  int c = 0;
 
   /* This parser is a true state machine, with no sneaky fetching
      of input characters inside the main loop.  In other words, all
@@ -411,9 +419,9 @@ compile (fp, filename, sections)
       octal,
       special_key
     }
-  seqstate;			/* used if state == get_keyseq */
+  seqstate;		/* used if state == get_keyseq */
   char meta = 0;
-  char ocnt;			/* used if state == get_keyseq && seqstate == octal */
+  char ocnt = 0;	/* used if state == get_keyseq && seqstate == octal */
 
   /* Data is accumulated in the following variables.  The code
      avoids overflowing these strings, and throws an error
@@ -421,17 +429,17 @@ compile (fp, filename, sections)
      lengths are arbitrary (and should be large enough) and their
      lengths are not hard-coded anywhere else, so increasing them
      here will not break anything.  */
-  char oval;
+  char oval = 0;
   char comment[10];
-  unsigned int clen;
+  unsigned int clen = 0;
   char seq[20];
-  unsigned int slen;
+  unsigned int slen = 0;
   char act[80];
-  unsigned int alen;
+  unsigned int alen = 0;
   char varn[80];
-  unsigned int varlen;
+  unsigned int varlen = 0;
   char val[80];
-  unsigned int vallen;
+  unsigned int vallen = 0;
 
 #define	To_seq(c) \
 		  do { \
@@ -439,7 +447,8 @@ compile (fp, filename, sections)
 		      seq[slen++] = meta ? Meta(c) : (c); \
 		    else \
 		      { \
-			syntax_error(filename, lnum, _("key sequence too long")); \
+			syntax_error(filename, lnum, _("key sequence too long"), \
+                            NULL, NULL, NULL, NULL); \
 			error = 1; \
 		      } \
 		    meta = 0; \
@@ -517,7 +526,8 @@ compile (fp, filename, sections)
 		  rescan = 1;
 		  if (slen == 0)
 		    {
-		      syntax_error (filename, lnum, _("missing key sequence"));
+		      syntax_error (filename, lnum, _("missing key sequence"),
+                          NULL, NULL, NULL, NULL);
 		      error = 1;
 		    }
 		}
@@ -596,7 +606,9 @@ compile (fp, filename, sections)
 		    To_seq (oval);
 		  else
 		    {
-		      syntax_error (filename, lnum, _("NUL character (\\000) not permitted"));
+		      syntax_error (filename, lnum,
+                          _("NUL character (\\000) not permitted"),
+                          NULL, NULL, NULL, NULL);
 		      error = 1;
 		    }
 		}
@@ -625,7 +637,9 @@ compile (fp, filename, sections)
 		To_seq (CONTROL (c));
 	      else
 		{
-		  syntax_error (filename, lnum, _("NUL character (^%c) not permitted"), c);
+		  syntax_error (filename, lnum,
+                      (char *) _("NUL character (^%c) not permitted"),
+                      (void *)((intptr_t)c), NULL, NULL, NULL);
 		  error = 1;
 		}
 	      seqstate = normal;
@@ -648,7 +662,8 @@ compile (fp, filename, sections)
 	      rescan = 1;
 	      if (alen == 0)
 		{
-		  syntax_error (filename, lnum, _("missing action name"), c);
+		  syntax_error (filename, lnum, (char *) _("missing action name"),
+                      (void *)((intptr_t)c), NULL, NULL, NULL);
 		  error = 1;
 		}
 	      else
@@ -663,13 +678,15 @@ compile (fp, filename, sections)
 			    && add_to_section (&sections[section], "", 1)
 			    && add_to_section (&sections[section], &av, 1)))
 			{
-			  syntax_error (filename, lnum, _("section too long"));
+			  syntax_error (filename, lnum, _("section too long"),
+                              NULL, NULL, NULL, NULL);
 			  error = 1;
 			}
 		    }
 		  else
 		    {
-		      syntax_error (filename, lnum, _("unknown action `%s'"), act);
+		      syntax_error (filename, lnum, _("unknown action `%s'"),
+                          act, NULL, NULL, NULL);
 		      error = 1;
 		    }
 		}
@@ -678,7 +695,8 @@ compile (fp, filename, sections)
 	    act[alen++] = c;
 	  else
 	    {
-	      syntax_error (filename, lnum, _("action name too long"));
+	      syntax_error (filename, lnum, _("action name too long"),
+                  NULL, NULL, NULL, NULL);
 	      error = 1;
 	    }
 	  break;
@@ -690,7 +708,9 @@ compile (fp, filename, sections)
 	    state = start_of_line;
 	  else if (!isspace (c))
 	    {
-	      syntax_error (filename, lnum, _("extra characters following action `%s'"), act);
+	      syntax_error (filename, lnum,
+                  _("extra characters following action `%s'"),
+                  act, NULL, NULL, NULL);
 	      error = 1;
 	    }
 	  break;
@@ -700,7 +720,8 @@ compile (fp, filename, sections)
 	    {
 	      if (varlen == 0)
 		{
-		  syntax_error (filename, lnum, _("missing variable name"));
+		  syntax_error (filename, lnum, _("missing variable name"),
+                      NULL, NULL, NULL, NULL);
 		  error = 1;
 		}
 	      state = get_value;
@@ -708,14 +729,17 @@ compile (fp, filename, sections)
 	    }
 	  else if (c == '\n' || isspace (c))
 	    {
-	      syntax_error (filename, lnum, _("missing `=' immediately after variable name"));
+	      syntax_error (filename, lnum,
+                  _("missing `=' immediately after variable name"),
+                  NULL, NULL, NULL, NULL);
 	      error = 1;
 	    }
 	  else if (varlen < sizeof varn)
 	    varn[varlen++] = c;
 	  else
 	    {
-	      syntax_error (filename, lnum, _("variable name too long"));
+	      syntax_error (filename, lnum, _("variable name too long"),
+                  NULL, NULL, NULL, NULL);
 	      error = 1;
 	    }
 	  break;
@@ -729,7 +753,8 @@ compile (fp, filename, sections)
 		    && add_to_section (&sections[section], val, vallen)
 		    && add_to_section (&sections[section], "", 1)))
 		{
-		  syntax_error (filename, lnum, _("section too long"));
+		  syntax_error (filename, lnum, _("section too long"),
+                      NULL, NULL, NULL, NULL);
 		  error = 1;
 		}
 	    }
@@ -737,10 +762,16 @@ compile (fp, filename, sections)
 	    val[vallen++] = c;
 	  else
 	    {
-	      syntax_error (filename, lnum, _("value too long"));
+	      syntax_error (filename, lnum, _("value too long"),
+                  NULL, NULL, NULL, NULL);
 	      error = 1;
 	    }
 	  break;
+
+        case get_equals:
+        case got_equals:
+        case got_varname:
+          break;
 	}
     }
 
@@ -753,14 +784,11 @@ compile (fp, filename, sections)
    characters fit, or false if the section's size limit was exceeded.
  */
 static int
-add_to_section (s, str, len)
-     struct sect *s;
-     const char *str;
-     unsigned int len;
+add_to_section (struct sect *s, const char *str, unsigned int len)
 {
   if (s->cur + len > sizeof s->data)
     return 0;
-  strncpy (s->data + s->cur, str, len);
+  strncpy ((char *) s->data + s->cur, str, len);
   s->cur += len;
   return 1;
 }
@@ -769,8 +797,7 @@ add_to_section (s, str, len)
    auto-generated array in key.c.
  */
 static int
-lookup_action (actname)
-     const char *actname;
+lookup_action (const char *actname)
 {
   int i;
 
@@ -787,9 +814,7 @@ lookup_action (actname)
    in radix INFOKEY_RADIX.
  */
 static int
-putint (i, fp)
-     int i;
-     FILE *fp;
+putint (int i, FILE *fp)
 {
   return fputc (i % INFOKEY_RADIX, fp) != EOF
     && fputc ((i / INFOKEY_RADIX) % INFOKEY_RADIX, fp) != EOF;
@@ -799,10 +824,7 @@ putint (i, fp)
    empty, simply omit it.
  */
 static int
-putsect (s, code, fp)
-     struct sect *s;
-     int code;
-     FILE *fp;
+putsect (struct sect *s, int code, FILE *fp)
 {
   if (s->cur == 0)
     return 1;
@@ -814,9 +836,7 @@ putsect (s, code, fp)
 /* Write an entire infokey file, given an array containing its sections.
  */
 static int
-write_infokey_file (fp, sections)
-     FILE *fp;
-     struct sect sections[];
+write_infokey_file (FILE *fp, struct sect *sections)
 {
   /* Get rid of sections with no effect. */
   if (sections[info].cur == 1 && sections[info].data[0] == 0)
@@ -848,10 +868,8 @@ write_infokey_file (fp, sections)
 	progname: "filename", line N: message
  */
 static void
-error_message (error_code, fmt, a1, a2, a3, a4)
-     int error_code;
-     const char *fmt;
-     const void *a1, *a2, *a3, *a4;
+error_message (int error_code, const char *fmt,
+    const void *a1, const void *a2, const void *a3, const void *a4)
 {
   fprintf (stderr, "%s: ", program_name);
   fprintf (stderr, fmt, a1, a2, a3, a4);
@@ -864,11 +882,9 @@ error_message (error_code, fmt, a1, a2, a3, a4)
 	progname: message
  */
 static void
-syntax_error (filename, linenum, fmt, a1, a2, a3, a4)
-     const char *filename;
-     unsigned int linenum;
-     const char *fmt;
-     const void *a1, *a2, *a3, *a4;
+syntax_error (const char *filename,
+    unsigned int linenum, const char *fmt,
+    const void *a1, const void *a2, const void *a3, const void *a4)
 {
   fprintf (stderr, "%s: ", program_name);
   fprintf (stderr, _("\"%s\", line %u: "), filename, linenum);
@@ -878,14 +894,14 @@ syntax_error (filename, linenum, fmt, a1, a2, a3, a4)
 
 /* Produce a gentle rtfm. */
 static void
-suggest_help ()
+suggest_help (void)
 {
   fprintf (stderr, _("Try --help for more information.\n"));
 }
 
 /* Produce a scaled down description of the available options to Info. */
 static void
-short_help ()
+short_help (void)
 {
   printf (_("\
 Usage: %s [OPTION]... [INPUT-FILE]\n\
